if UCV == nil then UCV = {} end
UCV.AddonName = "UCV"
UCV.version = 1.0
UCV.active = false
UCV.isLocked = true
UCV.showLabels = false
UCV.showIcons = true
UCV.itemIds = {71200, 71239, 71198, 71199, 114893, 114890, 114891, 114894, 114895, 114892, 114889, 30165}
local EM = EVENT_MANAGER
local debugMode = false
local strF = string.format
local _

local UCV_LTF = LibStub("LibTableFunctions-1.0")
if (not UCV_LTF) then return end

UCV.defaults = {
	firstRun = true,
	posX = 130,
	posY = 180,
	sizeX = 230,
	sizeY = 288,
	active = true,
	isLocked = false,
	showLabels = true,
	showIcons = true,
	itemIds = {},
}

local function UCV_GetLink(id)
	return (id == nil and "" or strF("|H1:item:%d:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0:0|h|h", id))
end

local function UCV_GetIcon(id)
	if(id == nil) then return "" end
	local iconVal = GetItemLinkInfo(UCV_GetLink(id))
	return iconVal
end

local function UCV_GetQuantity(id)
	if(id == nil) then return "" end
	local qtyVal
	_, qtyVal, _, _, _, _, _, _ = GetItemInfo(5, id)
	return strF("%d", qtyVal)
end

local function UCV_FillLine(currLine, currItem)
	currLine.link = (currItem == nil and "" or currItem.link)
	currLine.icon:SetTexture(currItem == nil and nil or currItem.icon)
	currLine.icon:SetAlpha(currItem == nil and 0 or 1)
	currLine.name:SetText(currItem == nil and "" or currItem.name)
	currLine.qty:SetText(currItem == nil and "" or currItem.qty)
end

local function UCV_InitializeInventoryLines()
	local currLine, currData
	for i = 1, #UCV.itemIds do
		currLine = UCV_GUI_ListHolder.lines[i]
		currData = UCV_GUI_ListHolder.dataLines[i]
		
		UCV_FillLine(currLine, (currData ~= nil and currData or nil))
	end
end

local function UCV_RemoveLine(i)
	local record
	if(_G["UCV_CBItem_"..i] ~= nil) then record = GetControl("UCV_CBItem_", i) else return end
	record.icon = record:GetNamedChild("_Icon")
	record.name = record:GetNamedChild("_Name")
	record.qty = record:GetNamedChild("_Qty")
	
	record:GetNamedChild("Bg"):SetAlpha(0)
	record:GetNamedChild("_Icon"):SetAlpha(0)
	record:GetNamedChild("_Name"):SetText("")
	record:GetNamedChild("_Qty"):SetText("")
	
	record:SetMouseEnabled(false)
end

local function UCV_CreateLine(i, predecessor, parent)
	local record = ((_G["UCV_CBItem_"..i] ~= nil) and _G["UCV_CBItem_"..i] or CreateControlFromVirtual("UCV_CBItem_", parent, "UCV_SlotTemplate", i))
	record.icon = record:GetNamedChild("_Icon")
	record.name = record:GetNamedChild("_Name")
	record.qty = record:GetNamedChild("_Qty")
	
	record:GetNamedChild("Bg"):SetAlpha(0)
	record:GetNamedChild("_Icon"):SetAlpha(0)
	record:SetHidden(false)
	record:SetMouseEnabled(true)
	record:SetHeight("24")
	
	if i == 1 then
		record:SetAnchor(TOPLEFT, UCV_GUI_ListHolder, TOPLEFT, 0, 0)
		record:SetAnchor(TOPRIGHT, UCV_GUI_ListHolder, TOPRIGHT, 0, 0)
	else
		record:SetAnchor(TOPLEFT, predecessor, BOTTOMLEFT, 0, UCV_GUI_ListHolder.rowHeight)
		record:SetAnchor(TOPRIGHT, predecessor, BOTTOMRIGHT, 0, UCV_GUI_ListHolder.rowHeight)
	end
	record:SetParent(UCV_GUI_ListHolder)
	return record
end

local function UCV_CreateListHolder()
	UCV_GUI_ListHolder.dataLines = {}
	UCV_GUI_ListHolder.lines = {}
	local predecessor = nil
	
	for i=1, #UCV.itemIds do
		UCV_GUI_ListHolder.lines[i] = UCV_CreateLine(i, predecessor, UCV_GUI_ListHolder)
		predecessor = UCV_GUI_ListHolder.lines[i]
	end
	UCV:UpdateDataLines()
end

function UCV:UpdateDataLines()
	local dataLines = {}
	local itemLink = nil
	
	if(#UCV.itemIds > UCV_GUI_ListHolder:GetNumChildren()) then
		UCV_GUI_ListHolder.dataLines = {}
		UCV_GUI_ListHolder.lines = {}
		UCV_CreateListHolder()
		UCV_GUI_ListHolder:SetParent(UCV_GUI)
	elseif(#UCV.itemIds < UCV_GUI_ListHolder:GetNumChildren()) then
		for i = #UCV.itemIds + 1, UCV_GUI_ListHolder:GetNumChildren() do
			UCV_RemoveLine(i)
		end
	end
	
	if(#UCV.itemIds > 0)then
		for i = 1, #UCV.itemIds do
			itemLink = UCV_GetLink(UCV.itemIds[i])

			if (itemLink ~= "") then
				table.insert(dataLines, {
					link = itemLink,
					qty = UCV_GetQuantity(UCV.itemIds[i]),
					icon = UCV_GetIcon(UCV.itemIds[i]),
					name = UCV:GetName(UCV.itemIds[i])
				})
			end
		end
	end
	
	UCV_GUI_ListHolder.dataLines = dataLines
	UCV_GUI_ListHolder:SetParent(UCV_GUI)
	UCV_InitializeInventoryLines()
	
	UCV_GUI:SetHeight(#UCV.itemIds * 24 + UCV_GUI_Header:GetHeight() + 12)
end

local function UCV_RedText(text)	return "|cFF0000"..tostring(text).."|r" end
local function UCV_GreenText(text)	return "|c00FF00"..tostring(text).."|r" end
local function UCV_BlueText(text)	return "|c0000FF"..tostring(text).."|r" end

--[[
local function UCV_OnReticleHidden(eventCode, hidden)
	if(UCV.active) then
		if(hidden and UCV.isLocked) then
			UCV_GUI:SetHidden(true)
		else
			UCV_GUI:SetHidden(false)
			UCV:UpdateDataLines()
		end
	end
end

local function UCV_ForceUpdate(eventCode, craftSkill)
	if(active) then
		UCV:UpdateDataLines()
	end
end
]]--

local function UCV_InvUpdated(eventCode, bagId, slotId, isNewItem, itemSoundCategory, inventoryUpdateReason, stackCountChange)
	if(UCV.active) then
		UCV:UpdateDataLines()
	end
end

local function UCV_ForceVisible()
	if(UCV.active and UCV.isLocked) then
		UCV.isLocked = not UCV.isLocked
		
		if(UCV.active) then
			UCV_GUI:SetHidden(false)
			UCV:UpdateDataLines()
		end
		
		-- Write isLocked to saved vars.
		UCV.sVar.isLocked = UCV.isLocked
	else
		if(not UCV.active) then
			d(GetString(UCV_MSG_HIDDEN))
		end
	end
end

local function UCV_OnLock()
	if(UCV.active and not UCV.isLocked) then
		UCV.isLocked = not UCV.isLocked
		
		if(UCV.active) then
			UCV_GUI:SetHidden(false)
			UCV:UpdateDataLines()
		else
			UCV_GUI:SetHidden(true)
		end
		
		-- Write isLocked to saved vars.
		UCV.sVar.isLocked = UCV.isLocked
	elseif(UCV.active and UCV.isLocked) then
		d(GetString(UCV_MSG_LOCKED))
	else
		if(not UCV.active) then
			d(GetString(UCV_MSG_HIDDEN))
		end
	end
end

function UCV:ToggleActive()
	if(HasCraftBagAccess() and CanAnyItemsBeStoredInCraftBag(5)) then
		UCV.active = not UCV.active
		
		-- Write active to saved vars.
		UCV.sVar.active = UCV.active
		
		if(UCV.active) then
			UCV_GUI:SetHidden(false)
			UCV:UpdateDataLines()
		else
			UCV_GUI:SetHidden(true)
		end
	else
		UCV.active = false
		UCV_GUI:SetHidden(true)
		d(GetString(UCV_MSG_NO_CBAG))
	end
end

local function UCV_BadSlash()
	d(GetString(UCV_MSG_BAD_SLASH))
	d(GetString(UCV_MSG_CMD_TITLE))
	
	strF(GetString(UCV_MSG_CMD_OPTION_1), (UCV.active and GetString(UCV_MSG_HIDE) or GetString(UCV_MSG_SHOW)))
	d(GetString(UCV_MSG_CMD_OPTION_2))
	d(GetString(UCV_MSG_CMD_OPTION_3))
end

function UCV:SavePos()
	--Make sure the window is not off screen.
	local width, height = UCV_GUI:GetDimensions()
	local posX, posY = UCV_GUI:GetCenter()
	
	posX = ((posX < width/2) and width/2 or posX)
	posY = ((posY < height/2) and height/2 or posY)
	UCV_GUI:ClearAnchors()
	UCV_GUI:SetAnchor(CENTER, GuiRoot, TOPLEFT, posX, posY)
	
	--Save the position.
	UCV.sVar.posX, UCV.sVar.posY = UCV_GUI:GetCenter()
end

function UCV:SaveSize()
	UCV.sVar.sizeX = UCV_GUI:GetWidth()
	UCV.sVar.sizeY = UCV_GUI:GetHeight()
	UCV:SavePos()
end

function UCV:GetLabelFont()
	--One day...
end

function UCV:SetLabelFont(value)
	--Maybe...
end


local function UCV_OnStart()
	if(UCV.sVar.firstRun) then
		d(GetString(UCV_MSG_INIT))
		
		if(UCV.sVar == nil) then UCV.sVar = UCV_LTF:CopyTable(UCV.defaults) end
		if(UCV.sVar.itemIds == nil or UCV.sVar.itemIds == {} or #UCV.sVar.itemIds < 1) then UCV.sVar.itemIds = UCV_LTF:CopyTable(UCV.itemIds) end
		
		--Show the welcome message.
		d(GetString(UCV_MSG_HELP))
		
		UCV.sVar.firstRun = false
	end
	
	UCV_CreateListHolder()
	UCV:UpdateDataLines()
	UCV_GUI_Header:SetParent(UCV_GUI)
	UCV_GUI_Header_Label:SetParent(UCV_GUI_Header)
	UCV_GUI_ListHolder:SetParent(UCV_GUI)
	UCV_GUI_Header_Label:SetText(GetString(UCV_GUI_TITLE))
	
	if(UCV.active) then
		UCV_GUI:SetHidden(false)
	else
		UCV_GUI:SetHidden(true)
	end
end

local function UCV_Initialized(eventCode, addonName)
	if addonName ~= "UCV" then return end
	
	UCV.sVar = ZO_SavedVars:NewCharacterIdSettings("UCV_Settings", UCV.version, nil, UCV.defaults)
	ZO_CreateStringId("SI_BINDING_NAME_UCV_TOGGLE", "Show Craftbag Viewer")
	
	SLASH_COMMANDS["/ucv"] = function(keyWord)
		if(string.lower(keyWord) == "unlock") then
			UCV_ForceVisible()
		elseif(string.lower(keyWord) == "lock") then
			UCV_OnLock()
		elseif(keyWord == "") then
			UCV:ToggleActive()
		else
			UCV_BadSlash()
		end
	end
	
	-- Restore settings
	UCV.active = UCV.sVar.active
	UCV.isLocked = UCV.sVar.isLocked
	UCV.showLabels = UCV.sVar.showLabels
	UCV.showIcons = UCV.sVar.showIcons
	
	if(UCV.sVar.itemIds ~= nil and #UCV.sVar.itemIds > 0) then UCV.itemIds = UCV.sVar.itemIds end
	
	UCV:SetupMenu()
	
	local width, height = UCV_GUI:GetDimensions()
	local posX, posY = UCV.sVar.posX, UCV.sVar.posY
	posX = ((posX < width/2) and width/2 or posX)
	posY = ((posY < height/2) and height/2 or posY)
	UCV_GUI:ClearAnchors()
	UCV_GUI:SetAnchor(CENTER, GuiRoot, TOPLEFT, posX, posY)
	
	local length = 0
	for _, v in pairs(UCV.sVar.itemIds) do
		length = math.floor(math.log10(v) + 1) > length and math.floor(math.log10(v) + 1) or length
	end
	
	UCV_GUI:SetWidth(UCV.sVar.sizeX + (length > 5 and (length - 5) * 10 or 0))
	UCV_GUI:SetHeight(#UCV.itemIds * 24 + UCV_GUI_Header:GetHeight() + 12)
	UCV_OnStart()
	
	--EM:RegisterForEvent("UCV", EVENT_RETICLE_HIDDEN_UPDATE, UCV_OnReticleHidden)
	--EM:RegisterForEvent("UCV", EVENT_CRAFT_COMPLETED, UCV_ForceUpdate)
	
	EM:RegisterForEvent("UCV_Backpack", EVENT_INVENTORY_SINGLE_SLOT_UPDATE, UCV_InvUpdated)
	EM:AddFilterForEvent("UCV_Backpack", EVENT_INVENTORY_SINGLE_SLOT_UPDATE, REGISTER_FILTER_BAG_ID, BAG_BACKPACK)
	
	EM:RegisterForEvent("UCV_CraftBag", EVENT_INVENTORY_SINGLE_SLOT_UPDATE, UCV_InvUpdated)
	EM:AddFilterForEvent("UCV_CraftBag", EVENT_INVENTORY_SINGLE_SLOT_UPDATE, REGISTER_FILTER_BAG_ID, BAG_VIRTUAL)
	
	EM:UnregisterForEvent("UCV_Initialized", EVENT_ADD_ON_LOADED)
end

EM:RegisterForEvent("UCV_Initialized", EVENT_ADD_ON_LOADED, UCV_Initialized)
